@@ -292,7 +292,7 @@ handle_single_mode (mode_run *mr, char sign, char mode, char *nick,
 	if (strchr (serv->nick_modes, mode))
 	{
 		/* update the user in the userlist */
-		ul_update_entry (sess, /*nickname */ arg, mode, sign);
+		userlist_update_mode (sess, /*nickname */ arg, mode, sign);
 	} else
 	{
 		if (!is_324 && !sess->ignore_mode)
@@ -391,6 +391,10 @@ handle_single_mode (mode_run *mr, char sign, char mode, char *nick,
 	fe_update_mode_buttons (sess, mode, sign);
 
  genmode:
+	/* Received umode +e. If we're waiting to send JOIN then send now! */
+	if (mode == 'e' && sign == '+' && !serv->p_cmp (chan, serv->nick))
+		inbound_identified (serv);
+
 	if (!quiet)
 	{
 		if (*arg)
@@ -575,7 +579,7 @@ handle_mode (server * serv, char *word[], char *word_eol[],
 			break;
 		default:
 			argstr = "";
-			if (all_modes_have_args || mode_has_arg (serv, sign, *modes))
+			if ((all_modes_have_args || mode_has_arg (serv, sign, *modes)) && arg < (num_args+1))
 			{
 				arg++;
 				argstr = word[arg + offset];
@@ -648,6 +652,12 @@ inbound_005 (server * serv, char *word[])
 				fe_set_channel (serv->server_session);
 			}
 
+			/* use /NICKSERV */
+			if (strcasecmp (word[w] + 8, "RusNet") == 0)
+				serv->nickservtype = 1;
+			else if (strcasecmp (word[w] + 8, "UniBG") == 0)
+				serv->nickservtype = 3;
+
 		} else if (strncmp (word[w], "CASEMAPPING=", 12) == 0)
 		{
 			if (strcmp (word[w] + 12, "ascii") == 0)	/* bahamut */
