@@ -4,7 +4,6 @@
 extern const struct commands xc_cmds[];
 extern GSList *menu_list;
 
-int load_trans_table (char *full_path);
 int auto_insert (char *dest, int destlen, unsigned char *src, char *word[], char *word_eol[],
 				 char *a, char *c, char *d, char *h, char *n, char *s);
 int handle_command (session *sess, char *cmd, int check_spch);
@@ -16,5 +15,6 @@ void notc_msg (session *sess);
 void server_sendpart (server * serv, char *channel, char *reason);
 void server_sendquit (session * sess);
 int menu_streq (const char *s1, const char *s2, int def);
+void open_query (server *serv, char *nick, gboolean focus_existing);
 
 #endif
