@@ -66,117 +66,201 @@ log_close (session *sess)
 }
 
 static void
-log_create_filename (char *buf, char *servname, char *channame, char *netname)
+mkdir_p (char *dir)	/* like "mkdir -p" from a shell, FS encoding */
 {
-	char fn[256];
-	char *tmp, *dir, *sep;
-	int pathlen=510, c=0, mbl;
+	char *start = dir;
 
-	if (!rfc_casecmp (channame, servname))
+	/* the whole thing already exists? */
+	if (access (dir, F_OK) == 0)
+		return;
+
+	while (*dir)
 	{
-		channame = g_strdup ("server");
-	} else
-	{
-		sep = tmp = strdup (channame);
-		while (*tmp)
-		{
-			mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
-			if (mbl == 1)
-			{
-#ifndef WIN32
-				*tmp = rfc_tolower (*tmp);
-				if (*tmp == '/')
-#else
-				/* win32 can't handle filenames with \|/>< characters */
-				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
-					 *tmp == '>' || *tmp == '<'  || *tmp == ':' || *tmp == '\"' || *tmp == '*' || *tmp == '?')
-#endif
-					*tmp = '_';
-			}
-			tmp += mbl;
-		}
-
-		channame = g_filename_from_utf8 (sep, -1, 0, 0, 0);
-		free (sep);
-	}
-
-	snprintf (buf, 512, "%s/xchatlogs", get_xdir_fs ());
-	if (access (buf, F_OK) != 0)
 #ifdef WIN32
-		mkdir (buf);
+		if (dir != start && (*dir == '/' || *dir == '\\'))
 #else
-		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
+		if (dir != start && *dir == '/')
 #endif
-	auto_insert (fn, sizeof (fn), prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
-	g_free (channame);
-
-	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir_fs (), fn);
-
- 	/* The following code handles subdirectories in logpath and creates
- 	 * them, if they don't exist. Useful with logmasks like "%c/%y.log" in
- 	 * ~/.xchat/xchat.conf.
- 	 *                       -- patch by tvk <tvk@von-koch.com>
- 	 */
-
-	if (access (buf, F_OK) != 0)
-	{
-		snprintf (buf, 512, "%s/xchatlogs/", get_xdir_fs ());
-		pathlen -= strlen(buf);
- 
-		/* how many sub-directories do we have? */
-		sep = fn;
-		while (*sep++) if (*sep=='/') c++;
-			
-		if (c) {
- 			/* create each sub-directory */
-			sep = strdup(fn);
-			dir = strtok(sep, "/");
-		
-			while (c-- && dir != NULL) {
-				strncat (buf, dir, pathlen);
-				strcat (buf, "/");
-				pathlen -= strlen(dir);
- 		
-				if (access (buf, F_OK) != 0)
-#ifdef WIN32	
-					mkdir (buf);
-#else	
-					mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
-#endif	
-				dir = strtok(NULL, "/");
-			}
-
-			/* append the filename */
-			strncat (buf, strrchr(fn, '/')+1, pathlen);
-			free (sep);
-		} else {
-			strncat (buf, fn, pathlen);
+		{
+			*dir = 0;
+#ifdef WIN32
+			mkdir (start);
+#else
+			mkdir (start, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+			*dir = '/';
 		}
-
+		dir++;
 	}
 }
 
+static char *
+log_create_filename (char *channame)
+{
+	char *tmp, *ret;
+	int mbl;
+
+	ret = tmp = strdup (channame);
+	while (*tmp)
+	{
+		mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
+		if (mbl == 1)
+		{
+#ifndef WIN32
+			*tmp = rfc_tolower (*tmp);
+			if (*tmp == '/')
+#else
+			/* win32 can't handle filenames with \|/><:"*? characters */
+			if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
+				 *tmp == '>'  || *tmp == '<' || *tmp == ':' ||
+				 *tmp == '\"' || *tmp == '*' || *tmp == '?')
+#endif
+				*tmp = '_';
+		}
+		tmp += mbl;
+	}
+
+	return ret;
+}
+
+/* like strcpy, but % turns into %% */
+
+static char *
+log_escape_strcpy (char *dest, char *src, char *end)
+{
+	while (*src)
+	{
+		*dest = *src;
+		if (dest + 1 == end)
+			break;
+		dest++;
+		src++;
+
+		if (*src == '%')
+		{
+			if (dest + 1 == end)
+				break;
+			dest[0] = '%';
+			dest++;
+		}
+	}
+
+	dest[0] = 0;
+	return dest - 1;
+}
+
+/* substitutes %c %n %s into buffer */
+
+static void
+log_insert_vars (char *buf, int bufsize, char *fmt, char *c, char *n, char *s)
+{
+	char *end = buf + bufsize;
+
+	while (1)
+	{
+		switch (fmt[0])
+		{
+		case 0:
+			buf[0] = 0;
+			return;
+
+		case '%':
+			fmt++;
+			switch (fmt[0])
+			{
+			case 'c':
+				buf = log_escape_strcpy (buf, c, end);
+				break;
+			case 'n':
+				buf = log_escape_strcpy (buf, n, end);
+				break;
+			case 's':
+				buf = log_escape_strcpy (buf, s, end);
+				break;
+			default:
+				buf[0] = '%';
+				buf++;
+				buf[0] = fmt[0];
+				break;
+			}
+			break;
+
+		default:
+			buf[0] = fmt[0];
+		}
+		fmt++;
+		buf++;
+		/* doesn't fit? */
+		if (buf == end)
+		{
+			buf[-1] = 0;
+			return;
+		}
+	}
+}
+
+static char *
+log_create_pathname (char *servname, char *channame, char *netname)
+{
+	char fname[384];
+	char fnametime[384];
+	char *fs;
+	struct tm *tm;
+	time_t now;
+
+	if (!netname)
+		netname = "NETWORK";
+
+	/* first, everything is in UTF-8 */
+	if (!rfc_casecmp (channame, servname))
+		channame = strdup ("server");
+	else
+		channame = log_create_filename (channame);
+	log_insert_vars (fname, sizeof (fname), prefs.logmask, channame, netname, servname);
+	free (channame);
+
+	/* insert time/date */
+	now = time (NULL);
+	tm = localtime (&now);
+	strftime (fnametime, sizeof (fnametime), fname, tm);
+
+	/* create final path/filename */
+	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
+
+	/* now we need it in FileSystem encoding */
+	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+
+	/* create all the subdirectories */
+	if (fs)
+		mkdir_p (fs);
+
+	return fs;
+}
+
 static int
 log_open_file (char *servname, char *channame, char *netname)
 {
 	char buf[512];
 	int fd;
+	char *file;
 	time_t currenttime;
 
-	if (!netname)
-		netname = "NETWORK";
+	file = log_create_pathname (servname, channame, netname);
+	if (!file)
+		return -1;
 
-	log_create_filename (buf, servname, channame, netname);
 #ifdef WIN32
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
 #else
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, 0644);
 #endif
+	g_free (file);
+
 	if (fd == -1)
 		return -1;
 	currenttime = time (NULL);
 	write (fd, buf,
-			 snprintf (buf, 510, _("**** BEGIN LOGGING AT %s\n"),
+			 snprintf (buf, sizeof (buf), _("**** BEGIN LOGGING AT %s\n"),
 						  ctime (&currenttime)));
 
 	return fd;
@@ -241,10 +325,25 @@ log_write (session *sess, char *text)
 {
 	char *temp;
 	char *stamp;
+	char *file;
 	int len;
 
 	if (sess->logfd != -1 && prefs.logging)
 	{
+		/* change to a different log file? */
+		file = log_create_pathname (sess->server->servername, sess->channel,
+											 server_get_network (sess->server, FALSE));
+		if (file)
+		{
+			if (access (file, F_OK) != 0)
+			{
+				close (sess->logfd);
+				sess->logfd = log_open_file (sess->server->servername, sess->channel,
+													  server_get_network (sess->server, FALSE));
+			}
+			g_free (file);
+		}
+
 		if (prefs.timestamp_logs)
 		{
 			len = get_stamp_str (prefs.timestamp_log_format, time (0), &stamp);
@@ -254,7 +353,7 @@ log_write (session *sess, char *text)
 				g_free (stamp);
 			}
 		}
-		temp = strip_color (text, -1, 1, 1);
+		temp = strip_color (text, -1, STRIP_ALL);
 		len = strlen (temp);
 		write (sess->logfd, temp, len);
 		/* lots of scripts/plugins print without a \n at the end */
@@ -264,40 +363,142 @@ log_write (session *sess, char *text)
 	}
 }
 
+/* converts a CP1252/ISO-8859-1(5) hybrid to UTF-8                           */
+/* Features: 1. It never fails, all 00-FF chars are converted to valid UTF-8 */
+/*           2. Uses CP1252 in the range 80-9f because ISO doesn't have any- */
+/*              thing useful in this range and it helps us receive from mIRC */
+/*           3. The five undefined chars in CP1252 80-9f are replaced with   */
+/*              ISO-8859-15 control codes.                                   */
+/*           4. Handles 0xa4 as a Euro symbol ala ISO-8859-15.               */
+/*           5. Uses ISO-8859-1 (which matches CP1252) for everything else.  */
+/*           6. This routine measured 3x faster than g_convert :)            */
+
+static unsigned char *
+iso_8859_1_to_utf8 (unsigned char *text, int len, gsize *bytes_written)
+{
+	unsigned int idx;
+	unsigned char *res, *output;
+	static const unsigned short lowtable[] = /* 74 byte table for 80-a4 */
+	{
+	/* compressed utf-8 table: if the first byte's 0x20 bit is set, it
+	   indicates a 2-byte utf-8 sequence, otherwise prepend a 0xe2. */
+		0x82ac, /* 80 Euro. CP1252 from here on... */
+		0xe281, /* 81 NA */
+		0x809a, /* 82 */
+		0xe692, /* 83 */
+		0x809e, /* 84 */
+		0x80a6, /* 85 */
+		0x80a0, /* 86 */
+		0x80a1, /* 87 */
+		0xeb86, /* 88 */
+		0x80b0, /* 89 */
+		0xe5a0, /* 8a */
+		0x80b9, /* 8b */
+		0xe592, /* 8c */
+		0xe28d, /* 8d NA */
+		0xe5bd, /* 8e */
+		0xe28f, /* 8f NA */
+		0xe290, /* 90 NA */
+		0x8098, /* 91 */
+		0x8099, /* 92 */
+		0x809c, /* 93 */
+		0x809d, /* 94 */
+		0x80a2, /* 95 */
+		0x8093, /* 96 */
+		0x8094, /* 97 */
+		0xeb9c, /* 98 */
+		0x84a2, /* 99 */
+		0xe5a1, /* 9a */
+		0x80ba, /* 9b */
+		0xe593, /* 9c */
+		0xe29d, /* 9d NA */
+		0xe5be, /* 9e */
+		0xe5b8, /* 9f */
+		0xe2a0, /* a0 */
+		0xe2a1, /* a1 */
+		0xe2a2, /* a2 */
+		0xe2a3, /* a3 */
+		0x82ac  /* a4 ISO-8859-15 Euro. */
+	};
+
+	if (len == -1)
+		len = strlen (text);
+
+	/* worst case scenario: every byte turns into 3 bytes */
+	res = output = g_malloc ((len * 3) + 1);
+	if (!output)
+		return NULL;
+
+	while (len)
+	{
+		if (G_LIKELY (*text < 0x80))
+		{
+			*output = *text;	/* ascii maps directly */
+		}
+		else if (*text <= 0xa4)	/* 80-a4 use a lookup table */
+		{
+			idx = *text - 0x80;
+			if (lowtable[idx] & 0x2000)
+			{
+				*output++ = (lowtable[idx] >> 8) & 0xdf; /* 2 byte utf-8 */
+				*output = lowtable[idx] & 0xff;
+			}
+			else
+			{
+				*output++ = 0xe2;	/* 3 byte utf-8 */
+				*output++ = (lowtable[idx] >> 8) & 0xff;
+				*output = lowtable[idx] & 0xff;
+			}
+		}
+		else if (*text < 0xc0)
+		{
+			*output++ = 0xc2;
+			*output = *text;
+		}
+		else
+		{
+			*output++ = 0xc3;
+			*output = *text - 0x40;
+		}
+		output++;
+		text++;
+		len--;
+	}
+	*output = 0;	/* terminate */
+	*bytes_written = output - res;
+
+	return res;
+}
+
 char *
 text_validate (char **text, int *len)
 {
 	char *utf;
 	gsize utf_len;
-	GError *error = NULL;
 
 	/* valid utf8? */
 	if (g_utf8_validate (*text, *len, 0))
 		return NULL;
 
+#ifdef WIN32
+	if (GetACP () == 1252) /* our routine is better than iconv's 1252 */
+#else
 	if (prefs.utf8_locale)
+#endif
 		/* fallback to iso-8859-1 */
-		utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+		utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	else
 	{
 		/* fallback to locale */
 		utf = g_locale_to_utf8 (*text, *len, 0, &utf_len, NULL);
 		if (!utf)
-			utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+			utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	}
 
 	if (!utf) 
 	{
-		if (error)
-		{
-			*text = g_strdup_printf ("\0034ICONV\017 %s\n", error->message);
-			*len = strlen (*text);
-			g_error_free (error);
-		} else
-		{
-			*text = g_strdup ("%INVALID%");
-			*len = 9;
-		}
+		*text = g_strdup ("%INVALID%");
+		*len = 9;
 	} else
 	{
 		*text = utf;
@@ -1016,14 +1217,11 @@ pevent_load (char *filename)
 	int penum = 0;
 	char *ofs;
 
-	buf = malloc (1000);
 	if (filename == NULL)
-		snprintf (buf, 1000, "%s/pevents.conf", get_xdir_fs ());
-   else
-      safe_strcpy (buf, filename, 1000);
+		fd = xchat_open_file ("pevents.conf", O_RDONLY, 0, 0);
+	else
+		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 
-	fd = open (buf, O_RDONLY | OFLAGS);
-	free (buf);
 	if (fd == -1)
 		return 1;
 	if (fstat (fd, &st) != 0)
@@ -1136,8 +1334,15 @@ load_text_events ()
 	pevent_make_pntevts ();
 }
 
+/*
+	CL: format_event now handles filtering of arguments:
+	1) if prefs.stripcolor is set, filter all style control codes from arguments
+	2) always strip \010 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
+*/
+#define ARG_FLAG(argn) (1 << (argn))
+
 void
-format_event (session *sess, int index, char **args, char *o, int sizeofo)
+format_event (session *sess, int index, char **args, char *o, int sizeofo, unsigned int stripcolor_args)
 {
 	int len, oi, ii, numargs;
 	char *i, *ar, d, a, done_all = FALSE;
@@ -1184,8 +1389,8 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 				printf ("arg[%d] is NULL in print event\n", a + 1);
 			} else
 			{
-				len = strlen (ar);
-				memcpy (&o[oi], ar, len);
+				if (stripcolor_args & ARG_FLAG(a + 1)) len = strip_color2 (ar, -1, &o[oi], STRIP_ALL);
+				else len = strip_hidden_attribute (ar, &o[oi]);
 				oi += len;
 			}
 			break;
@@ -1217,10 +1422,10 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 }
 
 static void
-display_event (session *sess, int event, char **args)
+display_event (session *sess, int event, char **args, unsigned int stripcolor_args)
 {
 	char o[4096];
-	format_event (sess, event, args, o, sizeof (o));
+	format_event (sess, event, args, o, sizeof (o), stripcolor_args);
 	if (o[0])
 		PrintText (sess, o);
 }
@@ -1401,6 +1606,25 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
+
+/* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
+/* also light/dark gray (14/15) */
+/* 5,7,8 are all shades of yellow which happen to look dman near the same */
+
+static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
+
+static int
+color_of (char *name)
+{
+	int i = 0, sum = 0;
+
+	while (name[i])
+		sum += name[i++];
+	sum %= sizeof (rcolors) / sizeof (char);
+	return rcolors[sum];
+}
+
+
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1408,21 +1632,21 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 {
 	char *word[PDIWORDS];
 	int i;
+	unsigned int stripcolor_args = (prefs.stripcolor ? 0xFFFFFFFF : 0);
+	char tbuf[NICKLEN + 4];
 
-	if (!a)
-		a = "\000";
-	if (!b)
-		b = "\000";
-	if (!c)
-		c = "\000";
-	if (!d)
-		d = "\000";
+	if (prefs.colorednicks && (index == XP_TE_CHANACTION || index == XP_TE_CHANMSG))
+	{
+		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (a), a);
+		a = tbuf;
+		stripcolor_args &= ~ARG_FLAG(1);	/* don't strip color from this argument */
+	}
 
 	word[0] = te[index].name;
-	word[1] = a;
-	word[2] = b;
-	word[3] = c;
-	word[4] = d;
+	word[1] = (a ? a : "\000");
+	word[2] = (b ? b : "\000");
+	word[3] = (c ? c : "\000");
+	word[4] = (d ? d : "\000");
 	for (i = 5; i < PDIWORDS; i++)
 		word[i] = "\000";
 
@@ -1433,7 +1657,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	/* If a plugin's callback executes "/close", 'sess' may be invalid */
 	if (is_session (sess))
-		display_event (sess, index, word);
+		display_event (sess, index, word, stripcolor_args);
 }
 
 int
@@ -1458,10 +1682,11 @@ pevent_save (char *fn)
 	char buf[1024];
 
 	if (!fn)
-		snprintf (buf, sizeof (buf), "%s/pevents.conf", get_xdir_fs ());
+		fd = xchat_open_file ("pevents.conf", O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE);
 	else
-		safe_strcpy (buf, fn, sizeof (buf));
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+		fd = xchat_open_file (fn, O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+									 XOF_FULLPATH | XOF_DOMODE);
 	if (fd == -1)
 	{
 		/*
@@ -1525,7 +1750,7 @@ sound_find_command (void)
 }
 
 void
-sound_play (const char *file)
+sound_play (const char *file, gboolean quiet)
 {
 	char buf[512];
 	char wavfile[512];
@@ -1536,9 +1761,6 @@ sound_play (const char *file)
 	if (!file[0])
 		return;
 
-	memset (buf, 0, sizeof (buf));
-	memset (wavfile, 0, sizeof (wavfile));
-
 #ifdef WIN32
 	/* check for fullpath, windows style */
 	if (strlen (file) > 3 &&
@@ -1554,6 +1776,7 @@ sound_play (const char *file)
 	{
 		strncpy (wavfile, file, sizeof (wavfile));
 	}
+	wavfile[sizeof (wavfile) - 1] = 0;	/* ensure termination */
 
 	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
 	if (!file_fs)
@@ -1586,8 +1809,11 @@ sound_play (const char *file)
 
 	} else
 	{
-		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FE_MSG_ERROR);
+		if (!quiet)
+		{
+			snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
+			fe_message (buf, FE_MSG_ERROR);
+		}
 	}
 
 	g_free (file_fs);
@@ -1597,7 +1823,7 @@ void
 sound_play_event (int i)
 {
 	if (sound_files[i])
-		sound_play (sound_files[i]);
+		sound_play (sound_files[i], FALSE);
 }
 
 static void
@@ -1622,8 +1848,7 @@ sound_load ()
 
 	memset (&sound_files, 0, sizeof (char *) * (NUM_XP));
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_RDONLY | OFLAGS);
+	fd = xchat_open_file ("sound.conf", O_RDONLY, 0, 0);
 	if (fd == -1)
 		return;
 
@@ -1653,8 +1878,8 @@ sound_save ()
 	int fd, i;
 	char buf[512];
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	fd = xchat_open_file ("sound.conf", O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+								 XOF_DOMODE);
 	if (fd == -1)
 		return;
 
