@@ -53,6 +53,10 @@
 #include "ssl.h"
 #endif
 
+#ifdef USE_MSPROXY
+#include "msproxy.h"
+#endif
+
 GSList *popup_list = 0;
 GSList *button_list = 0;
 GSList *dlgbutton_list = 0;
@@ -245,6 +249,9 @@ static int
 xchat_misc_checks (void)		/* this gets called every 1/2 second */
 {
 	static int count = 0;
+#ifdef USE_MSPROXY
+	static int count2 = 0;
+#endif
 
 	count++;
 
@@ -260,6 +267,15 @@ xchat_misc_checks (void)		/* this gets called every 1/2 second */
 		count = 0;
 	}
 
+#ifdef USE_MSPROXY	
+	count2++;
+	if (count2 >= 720)			/* 720 every 6 minutes */
+	{
+		msproxy_keepalive ();
+		count2 = 0;
+	}
+#endif
+
 	return 1;
 }
 
@@ -918,7 +934,8 @@ xchat_init (void)
 
 	servlist_init ();							/* load server list */
 
-	if (!prefs.slist_skip)
+	/* if we got a URL, don't open the server list GUI */
+	if (!prefs.slist_skip && !arg_url)
 		fe_serverlist_open (NULL);
 
 	/* turned OFF via -a arg */
@@ -937,7 +954,7 @@ xchat_init (void)
 		}
 	} else
 	{
-		if (prefs.slist_skip)
+		if (prefs.slist_skip || arg_url)
 			new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	}
 }
