@@ -42,6 +42,8 @@
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/util.h"
+#include "../common/fe.h"
+#include "../common/server.h"
 #include "gtkutil.h"
 #include "maingui.h"
 
@@ -296,7 +298,7 @@ chanlist_do_refresh (struct server *serv)
 
 		handle_command (serv->server_session, "list", FALSE);
 	} else
-		gtkutil_simpledialog ("Not connected.");
+		fe_message (_("Not connected."), FE_MSG_ERROR);
 }
 
 static void
@@ -363,7 +365,7 @@ fe_add_chan_list (struct server *serv, char *chan, char *users, char *topic)
 	next_row = (gchar **) malloc (sizeof (gchar *) * 3);
 	next_row[0] = strdup (chan);
 	next_row[1] = strdup (users);
-	next_row[2] = strip_color (topic);
+	next_row[2] = strip_color (topic, -1, STRIP_ALL);
 
 	/* add this row to the data */
 	chanlist_data_prepend_row (serv, next_row);
@@ -372,6 +374,12 @@ fe_add_chan_list (struct server *serv, char *chan, char *users, char *topic)
 	chanlist_place_row_in_gui (serv, next_row);
 }
 
+void
+fe_chan_list_end (struct server *serv)
+{
+	gtk_widget_set_sensitive (serv->gui->chanlist_refresh, TRUE);
+}
+
 /**
  * The next several functions simply handle signals from widgets to update 
  * the list and state variables. 
@@ -506,7 +514,7 @@ chanlist_filereq_done (struct server *serv, char *file)
 	if (fh == -1)
 		return;
 
-	snprintf (buf, sizeof buf, "X-Chat Channel List: %s - %s\n",
+	snprintf (buf, sizeof buf, "XChat Channel List: %s - %s\n",
 				 serv->servername, ctime (&t));
 	write (fh, buf, strlen (buf));
 
@@ -533,7 +541,7 @@ chanlist_save (GtkWidget * wid, struct server *serv)
 	if (!gtk_clist_get_text
 		 (GTK_CLIST (serv->gui->chanlist_list), 0, 0, &temp))
 	{
-		gtkutil_simpledialog (_("I can't save an empty list!"));
+		fe_message (_("I can't save an empty list!"), FE_MSG_ERROR);
 		return;
 	}
 	gtkutil_file_req (_("Select an output filename"), chanlist_filereq_done,
@@ -588,7 +596,7 @@ chanlist_closegui (GtkWidget *wid, server *serv)
 }
 
 void
-chanlist_opengui (struct server *serv)
+chanlist_opengui (struct server *serv, int do_refresh)
 {
 	gchar *titles[3];
 	GtkWidget *frame, *vbox, *hbox, *table, *wid;
@@ -604,7 +612,7 @@ chanlist_opengui (struct server *serv)
 		return;
 	}
 
-	snprintf (tbuf, sizeof tbuf, _("X-Chat: Channel List (%s)"),
+	snprintf (tbuf, sizeof tbuf, _("XChat: Channel List (%s)"),
 				 serv->servername);
 
 	serv->gui->chanlist_data_stored_rows = NULL;
@@ -616,7 +624,7 @@ chanlist_opengui (struct server *serv)
 		serv->gui->chanlist_maxusers = 9999;
 
 	serv->gui->chanlist_window =
-		mg_create_generic_tab ("chanlist", tbuf, FALSE, TRUE, chanlist_closegui,
+		mg_create_generic_tab ("ChanList", tbuf, FALSE, TRUE, chanlist_closegui,
 								serv, 450, 356, &vbox, serv);
 
 	frame = gtk_frame_new (_("List display options:"));
@@ -640,7 +648,7 @@ chanlist_opengui (struct server *serv)
 	wid = gtk_spin_button_new_with_range (0, 999999, 1);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
 										serv->gui->chanlist_minusers);
-	g_signal_connect (G_OBJECT (wid), "value-changed",
+	g_signal_connect (G_OBJECT (wid), "value_changed",
 							G_CALLBACK (chanlist_minusers), serv);
 	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, 0, 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
@@ -655,13 +663,17 @@ chanlist_opengui (struct server *serv)
 	wid = gtk_spin_button_new_with_range (0, 999999, 1);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
 										serv->gui->chanlist_maxusers);
-	g_signal_connect (G_OBJECT (wid), "value-changed",
+	g_signal_connect (G_OBJECT (wid), "value_changed",
 							G_CALLBACK (chanlist_maxusers), serv);
 	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, 2, 3,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 	gtk_widget_show (wid);
 
+#ifdef WIN32
+	wid = gtk_label_new (_("Pattern Match:"));
+#else
 	wid = gtk_label_new (_("Regex Match:"));
+#endif
 	gtk_misc_set_alignment (GTK_MISC (wid), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, 0, 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
@@ -749,7 +761,7 @@ chanlist_opengui (struct server *serv)
 	serv->gui->chanlist_refresh =
 		gtkutil_button (hbox, GTK_STOCK_REFRESH, 0, chanlist_refresh, serv,
 							_("Refresh the list"));
-	gtkutil_button (hbox, GTK_STOCK_SAVE, 0, chanlist_save, serv,
+	gtkutil_button (hbox, GTK_STOCK_SAVE_AS, 0, chanlist_save, serv,
 						_("Save the list"));
 	gtkutil_button (hbox, GTK_STOCK_JUMP_TO, 0, chanlist_join, serv,
 						 _("Join Channel"));
@@ -765,4 +777,7 @@ chanlist_opengui (struct server *serv)
 	chanlist_reset_sort_vars (serv);
 
 	gtk_widget_show (serv->gui->chanlist_window);
+
+	if (do_refresh)
+		chanlist_do_refresh (serv);
 }
