@@ -32,6 +32,7 @@
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtkmessagedialog.h>
 
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
@@ -52,15 +53,15 @@
 #define MEGABYTE (KILOBYTE * 1024)
 #define GIGABYTE (MEGABYTE * 1024)
 
-static void proper_unit (unsigned int size, char *buf, int buf_len)
+static void proper_unit (DCC_SIZE size, char *buf, int buf_len)
 {
 	if (size <= KILOBYTE)
 	{
-		snprintf (buf, buf_len, "%uB", size);
+		snprintf (buf, buf_len, "%"DCC_SFMT"B", size);
 	}
 	else if (size > KILOBYTE && size <= MEGABYTE)
 	{
-		snprintf (buf, buf_len, "%ukB", size / KILOBYTE);
+		snprintf (buf, buf_len, "%"DCC_SFMT"kB", size / KILOBYTE);
 	}
 	else
 	{
@@ -81,6 +82,7 @@ struct my_dcc_send
 	struct session *sess;
 	char *nick;
 	int maxcps;
+	int passive;
 };
 
 static struct dccwindow dccrwin;	/* recv */
@@ -92,19 +94,7 @@ static void
 dcc_send_filereq_file (struct my_dcc_send *mdc, char *file)
 {
 	if (file)
-		dcc_send (mdc->sess, mdc->nick, file, mdc->maxcps, 0);
-	else
-	{
-		free (mdc->nick);
-		free (mdc);
-	}
-}
-
-static void
-dcc_psend_filereq_file (struct my_dcc_send *mdc, char *file)
-{
-	if (file)
-		dcc_send (mdc->sess, mdc->nick, file, mdc->maxcps, 1);
+		dcc_send (mdc->sess, mdc->nick, file, mdc->maxcps, mdc->passive);
 	else
 	{
 		free (mdc->nick);
@@ -122,12 +112,10 @@ fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps, int passive)
 	mdc->sess = sess;
 	mdc->nick = strdup (nick);
 	mdc->maxcps = maxcps;
+	mdc->passive = passive;
 
 	snprintf (tbuf, sizeof tbuf, _("Send file to %s"), nick);
-	if (passive)
-		gtkutil_file_req (tbuf, dcc_psend_filereq_file, mdc, NULL, FRF_MULTIPLE);
-	else
-		gtkutil_file_req (tbuf, dcc_send_filereq_file, mdc, NULL, FRF_MULTIPLE);
+	gtkutil_file_req (tbuf, dcc_send_filereq_file, mdc, NULL, FRF_MULTIPLE);
 }
 
 static void
@@ -310,6 +298,7 @@ dcc_update_recv_win (void)
 	GSList *list = dcc_list;
 	gchar *nnew[9];
 	gint row;
+	int i = 0;
 
 	gtk_clist_freeze (GTK_CLIST (dccrwin.list));
 	gtk_clist_clear (GTK_CLIST (dccrwin.list));
@@ -326,30 +315,50 @@ dcc_update_recv_win (void)
 				gtk_clist_set_foreground (GTK_CLIST (dccrwin.list), row,
 												  colors +
 												  dccstat[dcc->dccstat].color);
+			i++;
 		}
 		list = list->next;
 	}
+
+	/* if only one entry, select it (so Accept button can work) */
+	if (i == 1)
+		gtk_clist_select_row (GTK_CLIST (dccrwin.list), 0, 0);
+
 	gtk_clist_thaw (GTK_CLIST (dccrwin.list));
 }
 
 static void
 dcc_info (struct DCC *dcc)
 {
-	char tbuf[256];
-	snprintf (tbuf, 255, _("      File: %s\n"
-				 "   To/From: %s\n"
-				 "      Size: %u\n"
-				 "      Port: %d\n"
-				 " IP Number: %s\n"
-				 "Start Time: %s"
-				 "   Max CPS: %d\n"),
-				 dcc->file,
-				 dcc->nick,
-				 dcc->size,
-				 dcc->port,
-				 net_ip (dcc->addr), ctime (&dcc->starttime),
-				 dcc->maxcps);
-	gtkutil_simpledialog (tbuf);
+	char max[48];
+	char siz[48];
+	GtkWidget *dialog;
+
+	if (dcc->maxcps)
+		sprintf (max, "%.2f KB/s", (float)dcc->maxcps / 1024.0);
+	else
+		snprintf (max, sizeof (max), "%s", _("None"));
+
+	proper_unit (dcc->size, siz, sizeof (siz));
+
+	dialog = gtk_message_dialog_new_with_markup (NULL, 0, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+				 "<tt><b>%-13s:</b></tt> %s\n"
+				 "<tt><b>%-13s:</b></tt> %s\n"
+				 "<tt><b>%-13s:</b></tt> %s (%"DCC_SFMT" bytes)\n"
+				 "<tt><b>%-13s:</b></tt> %s : %d\n"
+				 "<tt><b>%-13s:</b></tt> %s"
+				 "<tt><b>%-13s:</b></tt> %s\n",
+				 _("File"), (dcc->type == TYPE_RECV) ? dcc->destfile : dcc->file,
+				 (dcc->type == TYPE_RECV) ? _("From") : _("To"), dcc->nick,
+				 _("Size"), siz, dcc->size,
+				 _("Address"), net_ip (dcc->addr), dcc->port,
+				 _("Started"), ctime (&dcc->starttime),
+				 _("Speed limit"), max);
+	g_signal_connect (G_OBJECT (dialog), "response",
+							G_CALLBACK (gtk_widget_destroy), 0);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -369,7 +378,7 @@ resume_clicked (GtkWidget * wid, gpointer none)
 			switch (dcc->resume_error)
 			{
 			case 0:	/* unknown error */
-				gtkutil_simpledialog (_("That file is not resumable."));
+				fe_message (_("That file is not resumable."), FE_MSG_ERROR);
 				break;
 			case 1:
 				snprintf (buf, sizeof (buf),
@@ -377,14 +386,14 @@ resume_clicked (GtkWidget * wid, gpointer none)
 								"%s.\n"
 								"Resuming not possible."), dcc->destfile,	
 								errorstring (dcc->resume_errno));
-				gtkutil_simpledialog (buf);
+				fe_message (buf, FE_MSG_ERROR);
 				break;
 			case 2:
-				gtkutil_simpledialog (_("File in download directory is larger "
-											"than file offered. Resuming not possible."));
+				fe_message (_("File in download directory is larger "
+								"than file offered. Resuming not possible."), FE_MSG_ERROR);
 				break;
 			case 3:
-				gtkutil_simpledialog (_("Cannot resume the same file from two people."));
+				fe_message (_("Cannot resume the same file from two people."), FE_MSG_ERROR);
 			}
 		}
 	}
@@ -393,30 +402,46 @@ resume_clicked (GtkWidget * wid, gpointer none)
 static void
 abort_clicked (GtkWidget * wid, gpointer none)
 {
-	int row;
+	int * rows;
+	int i, num_rows;
 	struct DCC *dcc;
 
-	row = gtkutil_clist_selection (dccrwin.list);
-	if (row != -1)
+	num_rows = gtkutil_clist_multiple_selection (dccrwin.list, &rows, 256);
+	gtk_clist_freeze( GTK_CLIST(dccrwin.list) );
+
+	for (i = num_rows - 1; i >= 0; i--)
 	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), row);
-		dcc_abort (dcc->serv->front_session, dcc);
+		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), rows[i]);
+		if ( dcc )
+		{
+			dcc_abort (dcc->serv->front_session, dcc);
+		}
 	}
+	gtk_clist_thaw( GTK_CLIST(dccrwin.list) );
+	free ( rows );
 }
 
 static void
 accept_clicked (GtkWidget * wid, gpointer none)
 {
-	int row;
+	int * rows;
+	int i, num_rows;
 	struct DCC *dcc;
 
-	row = gtkutil_clist_selection (dccrwin.list);
-	if (row != -1)
+	num_rows = gtkutil_clist_multiple_selection (dccrwin.list, &rows, 256);
+	gtk_clist_freeze( GTK_CLIST(dccrwin.list) );
+
+	for (i = num_rows - 1; i >= 0; i--)
 	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), row);
-		gtk_clist_unselect_row (GTK_CLIST (dccrwin.list), row, 0);
-		dcc_get (dcc);
+		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), rows[i]);
+		if ( dcc )
+		{
+			gtk_clist_unselect_row (GTK_CLIST (dccrwin.list), rows[i], 0);
+			dcc_get (dcc);
+		}
 	}
+	gtk_clist_thaw( GTK_CLIST(dccrwin.list) );
+	free ( rows );
 }
 
 static void
@@ -463,15 +488,44 @@ recv_row_selected (GtkWidget * clist, gint row, gint column,
 		accept_clicked (0, 0);
 }
 
+static void
+browse_folder (char *dir)
+{
+#ifdef WIN32
+	/* no need for file:// in ShellExecute() */
+	fe_open_url (dir);
+#else
+	char buf[512];
+
+	snprintf (buf, sizeof (buf), "file://%s", dir);
+	fe_open_url (buf);
+#endif
+}
+
+static gboolean
+recv_key_press (GtkWidget *wid, GdkEventKey *evt, gpointer unused)
+{
+	if (evt->state & GDK_CONTROL_MASK && evt->keyval == 0x06f)
+	{
+		if (prefs.dcc_completed_dir[0])
+			browse_folder (prefs.dcc_completed_dir);
+		else
+			browse_folder (prefs.dccdir);
+	}
+
+	return FALSE;
+}
+
 int
 fe_dcc_open_recv_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
+	int i;
 #ifdef USE_GNOME
 	gchar *titles[] =
 		{ NULL, NULL, NULL, NULL, "%", "KB/s", NULL, NULL, NULL };
 
-	titles[8] = _("MIME Type");
+	titles[8] = "MIME Type";
 #else
 	gchar *titles[] =
 		{ NULL, NULL, NULL, NULL, "%", "KB/s", NULL, NULL };
@@ -491,26 +545,30 @@ fe_dcc_open_recv_win (int passive)
 		return TRUE;
 	}
 	dccrwin.window =
-			  mg_create_generic_tab ("dccrecv", _("X-Chat: File Receive List"),
-						FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0, &vbox, 0);
-
+			  mg_create_generic_tab ("Downloads", _("XChat: File Receive List"),
+						FALSE, TRUE, close_dcc_recv_window, NULL, 601, 200, &vbox, 0);
+	if (!prefs.windows_as_tabs)
+		gtk_window_set_position (GTK_WINDOW (dccrwin.window), GTK_WIN_POS_NONE);
+	g_signal_connect (G_OBJECT (dccrwin.window), "key_release_event",
+							G_CALLBACK (recv_key_press), 0);
 #ifdef USE_GNOME
 	dccrwin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
 												 recv_row_selected, 0,
 												 0, 0, GTK_SELECTION_SINGLE);
+	for(i=0; i < 9; i++)
+	{
+		gtk_clist_set_column_auto_resize(GTK_CLIST(dccrwin.list), i, TRUE);
+	}
 #else
 	dccrwin.list = gtkutil_clist_new (8, titles, vbox, GTK_POLICY_ALWAYS,
 												 recv_row_selected, 0,
 												 0, 0, GTK_SELECTION_SINGLE);
+	for(i=0; i < 8; i++)
+	{
+		gtk_clist_set_column_auto_resize(GTK_CLIST(dccrwin.list), i, TRUE);
+	}
 #endif
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 0, 65);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 1, 100);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 2, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 3, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 4, 30);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 5, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 6, 60);
-	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 7, 60);
+	gtk_clist_set_selection_mode(GTK_CLIST(dccrwin.list), GTK_SELECTION_MULTIPLE);
 	gtk_clist_set_column_justification (GTK_CLIST (dccrwin.list), 4,
 													GTK_JUSTIFY_CENTER);
 
@@ -548,6 +606,7 @@ dcc_update_send_win (void)
 	GSList *list = dcc_list;
 	gchar *nnew[9];
 	gint row;
+	int i = 0;
 
 	gtk_clist_freeze (GTK_CLIST (dccswin.list));
 	gtk_clist_clear ((GtkCList *) dccswin.list);
@@ -563,9 +622,15 @@ dcc_update_send_win (void)
 				gtk_clist_set_foreground
 					(GTK_CLIST (dccswin.list), row,
 					 colors + dccstat[dcc->dccstat].color);
+			i++;
 		}
 		list = list->next;
 	}
+
+	/* if only one entry, select it (so Abort button can work) */
+	if (i == 1)
+		gtk_clist_select_row (GTK_CLIST (dccswin.list), 0, 0);
+
 	gtk_clist_thaw (GTK_CLIST (dccswin.list));
 }
 
@@ -609,21 +674,32 @@ info_send_clicked (GtkWidget * wid, gpointer none)
 static void
 abort_send_clicked (GtkWidget * wid, gpointer none)
 {
-	int row;
+	int * rows;
+	int i, num_rows;
 	struct DCC *dcc;
 
-	row = gtkutil_clist_selection (dccswin.list);
-	if (row != -1)
+	num_rows = gtkutil_clist_multiple_selection (dccswin.list, &rows, 256);
+	gtk_clist_freeze( GTK_CLIST(dccswin.list) );
+
+	/* we go from the end to the begining, because if we remove a row it might affect
+	the rows that's coming after it */
+	for (i = num_rows - 1; i >= 0; i--)
 	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dccswin.list), row);
-		dcc_abort (dcc->serv->front_session, dcc);
+		dcc = gtk_clist_get_row_data (GTK_CLIST (dccswin.list), rows[i]);
+		if ( dcc )
+		{
+			dcc_abort (dcc->serv->front_session, dcc);
+		}
 	}
+	gtk_clist_thaw( GTK_CLIST(dccswin.list) );
+	free ( rows );
 }
 
 int
 fe_dcc_open_send_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
+	int i;
 	gchar *titles[] =
 		{ NULL, NULL, NULL, NULL, NULL, "%", "KB/s", NULL, NULL };
 
@@ -642,21 +718,28 @@ fe_dcc_open_send_win (int passive)
 		return TRUE;
 	}
 
+#if 0
+	N_("XChat: Uploads und Downloads"),
+	N_("Downloads"),
+	N_("Uploads"),
+	N_("Details"),
+	N_("Open Folder..."),
+#endif
+
 	dccswin.window =
-			  mg_create_generic_tab ("dccsend", _("X-Chat: File Send List"),
-						FALSE, TRUE, close_dcc_send_window, NULL, 595, 0, &vbox, 0);
+			  mg_create_generic_tab ("Uploads", _("XChat: File Send List"),
+						FALSE, TRUE, close_dcc_send_window, NULL, 595, 200, &vbox, 0);
 
 	dccswin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
 												 send_row_selected, 0,
 												 0, 0, GTK_SELECTION_SINGLE);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 0, 65);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 1, 100);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 2, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 3, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 4, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 5, 30);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 6, 50);
-	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 7, 50);
+
+	for(i=0; i < 9; i++)
+	{
+		gtk_clist_set_column_auto_resize(GTK_CLIST(dccswin.list), i, TRUE);
+	}
+	gtk_clist_set_selection_mode(GTK_CLIST(dccswin.list), GTK_SELECTION_MULTIPLE);
+
 	gtk_clist_set_column_justification (GTK_CLIST (dccswin.list), 5,
 													GTK_JUSTIFY_CENTER);
 
@@ -680,30 +763,50 @@ fe_dcc_open_send_win (int passive)
 static void
 accept_chat_clicked (GtkWidget * wid, gpointer none)
 {
-	int row;
+	int * rows;
+	int i, num_rows;
 	struct DCC *dcc;
 
-	row = gtkutil_clist_selection (dcccwin.list);
-	if (row != -1)
+	num_rows = gtkutil_clist_multiple_selection (dcccwin.list, &rows, 256);
+	gtk_clist_freeze( GTK_CLIST(dcccwin.list) );
+
+	/* we go from the end to the begining, because if we remove a row it might affect
+	the rows that's coming after it */
+	for (i = num_rows - 1; i >= 0; i--)
 	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), row);
-		gtk_clist_unselect_row (GTK_CLIST (dcccwin.list), row, 0);
-		dcc_get (dcc);
+		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), rows[i]);
+		if ( dcc )
+		{
+			gtk_clist_unselect_row (GTK_CLIST (dcccwin.list), rows[i], 0);
+			dcc_get (dcc);
+		}
 	}
+	gtk_clist_thaw( GTK_CLIST(dcccwin.list) );
+	free ( rows );
 }
 
 static void
 abort_chat_clicked (GtkWidget * wid, gpointer none)
 {
-	int row;
+	int * rows;
+	int i, num_rows;
 	struct DCC *dcc;
 
-	row = gtkutil_clist_selection (dcccwin.list);
-	if (row != -1)
+	num_rows = gtkutil_clist_multiple_selection (dcccwin.list, &rows, 256);
+	gtk_clist_freeze( GTK_CLIST(dcccwin.list) );
+
+	/* we go from the end to the begining, because if we remove a row it might affect
+	the rows that's coming after it */
+	for (i = num_rows - 1; i >= 0; i--)
 	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), row);
-		dcc_abort (dcc->serv->front_session, dcc);
+		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), rows[i]);
+		if ( dcc )
+		{
+			dcc_abort (dcc->serv->front_session, dcc);
+		}
 	}
+	gtk_clist_thaw( GTK_CLIST(dcccwin.list) );
+	free ( rows );
 }
 
 static void
@@ -749,12 +852,13 @@ fe_dcc_open_chat_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
 	gchar *titles[5];
+	int i;
 
 	titles[0] = _("Status");
 	titles[1] = _("To/From");
 	titles[2] = _("Recv");
 	titles[3] = _("Sent");
-	titles[4] = _("StartTime");
+	titles[4] = _("Start Time");
 
 	if (dcccwin.window)
 	{
@@ -764,16 +868,18 @@ fe_dcc_open_chat_win (int passive)
 	}
 
 	dcccwin.window =
-			  mg_create_generic_tab ("dccchat", _("X-Chat: DCC Chat List"),
-						FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0, &vbox, 0);
+			  mg_create_generic_tab ("DCCChat", _("XChat: DCC Chat List"),
+						FALSE, TRUE, close_dcc_chat_window, NULL, 550, 180, &vbox, 0);
 
 	dcccwin.list = gtkutil_clist_new (5, titles, vbox, GTK_POLICY_ALWAYS,
 												 chat_row_selected, 0,
 												 0, 0, GTK_SELECTION_BROWSE);
-	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 0, 65);
-	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 1, 100);
-	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 2, 65);
-	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 3, 65);
+
+	for(i=0; i < 5; i++)
+	{
+		gtk_clist_set_column_auto_resize(GTK_CLIST(dcccwin.list), i, TRUE);
+	}
+	gtk_clist_set_selection_mode(GTK_CLIST(dcccwin.list), GTK_SELECTION_MULTIPLE);
 
 	bbox = gtk_hbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
