@@ -157,7 +157,7 @@ fe_args (int argc, char *argv[])
 
 	if (arg_show_version)
 	{
-		printf (PACKAGE_NAME" "PACKAGE_VERSION"\n");
+		printf (PACKAGE_TARNAME" "PACKAGE_VERSION"\n");
 		return 0;
 	}
 
@@ -225,7 +225,11 @@ fe_args (int argc, char *argv[])
 const char cursor_color_rc[] =
 	"style \"xc-ib-st\""
 	"{"
+#ifdef USE_GTKSPELL
+		"GtkTextView::cursor-color=\"#%02x%02x%02x\""
+#else
 		"GtkEntry::cursor-color=\"#%02x%02x%02x\""
+#endif
 	"}"
 	"widget \"*.xchat-inputbox\" style : application \"xc-ib-st\"";
 
@@ -378,6 +382,8 @@ fe_message (char *msg, int flags)
 
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent_window), 0, type,
 												GTK_BUTTONS_OK, "%s", msg);
+	if (flags & FE_MSG_MARKUP)
+		gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), msg);
 	g_signal_connect (G_OBJECT (dialog), "response",
 							G_CALLBACK (gtk_widget_destroy), 0);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
@@ -726,10 +732,16 @@ fe_ctrl_gui (session *sess, int action, int arg)
 static void
 dcc_saveas_cb (struct DCC *dcc, char *file)
 {
-	if (file)
-		dcc_get_with_destfile (dcc, file);
-	else if (dcc->dccstat == STAT_QUEUED && dcc->resume_sent == 0)
-		dcc_abort (dcc->serv->front_session, dcc);
+	if (is_dcc (dcc))
+	{
+		if (dcc->dccstat == STAT_QUEUED)
+		{
+			if (file)
+				dcc_get_with_destfile (dcc, file);
+			else if (dcc->resume_sent == 0)
+				dcc_abort (dcc->serv->front_session, dcc);
+		}
+	}
 }
 
 void
