@@ -69,6 +69,10 @@
 #include <gtkspell/gtkspell.h>
 #endif
 
+#ifdef USE_LIBSEXY
+#include "sexy-spell-entry.h"
+#endif
+
 #define GUI_SPACING (3)
 #define GUI_BORDER (1)
 #define SCROLLBAR_SPACING (2)
@@ -150,7 +154,7 @@ SPELL_ENTRY_INSERT (GtkWidget *entry, const char *text, int len, int *pos)
 #endif
 
 static PangoAttrList *
-mg_attr_list_create (GdkColor *col, gboolean small)
+mg_attr_list_create (GdkColor *col, int size)
 {
 	PangoAttribute *attr;
 	PangoAttrList *list;
@@ -165,9 +169,9 @@ mg_attr_list_create (GdkColor *col, gboolean small)
 		pango_attr_list_insert (list, attr);
 	}
 
-	if (small)
+	if (size > 0)
 	{
-		attr = pango_attr_scale_new (PANGO_SCALE_SMALL);
+		attr = pango_attr_scale_new (size == 1 ? PANGO_SCALE_SMALL : PANGO_SCALE_X_SMALL);
 		attr->start_index = 0;
 		attr->end_index = 0xffff;
 		pango_attr_list_insert (list, attr);
@@ -2119,7 +2123,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 			return;
 		}
 
-		if (even->state & GDK_CONTROL_MASK)
+		if ((even->state & 13) == prefs.gui_url_mod)
 		{
 			switch (mg_word_check (xtext, word, strlen (word)))
 			{
@@ -2547,6 +2551,7 @@ mg_create_entry (session *sess, GtkWidget *box)
 	gui->input_box = entry = gtk_text_view_new ();
 	gtk_widget_set_size_request (entry, 0, 1);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (entry), GTK_WRAP_NONE);
+	gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (entry), FALSE);
 	if (prefs.gui_input_spell)
 		gtkspell_new_attach (GTK_TEXT_VIEW (entry), NULL, NULL);
 
@@ -2558,8 +2563,13 @@ mg_create_entry (session *sess, GtkWidget *box)
 												GTK_POLICY_NEVER);
 	gtk_container_add (GTK_CONTAINER (sw), entry);
 	gtk_container_add (GTK_CONTAINER (hbox), sw);
-#else	
+#else
+#ifdef USE_LIBSEXY
+	gui->input_box = entry = sexy_spell_entry_new ();
+	sexy_spell_entry_set_checked ((SexySpellEntry *)entry, prefs.gui_input_spell);
+#else
 	gui->input_box = entry = gtk_entry_new ();
+#endif
 	gtk_entry_set_max_length (GTK_ENTRY (gui->input_box), 2048);
 	g_signal_connect (G_OBJECT (entry), "activate",
 							G_CALLBACK (mg_inputbox_cb), gui);
@@ -3049,6 +3059,13 @@ fe_set_away (server *serv)
 	}
 }
 
+void
+fe_set_color_paste (session *sess, int status)
+{
+	sess->color_paste = status;
+	if (!sess->gui->is_tab || sess == current_tab)
+		GTK_XTEXT (sess->gui->xtext)->color_paste = status;
+}
 void
 fe_set_channel (session *sess)
 {
@@ -3210,6 +3227,9 @@ fe_server_callback (server *serv)
 void
 fe_session_callback (session *sess)
 {
+	if (sess->res->banlist_window)
+		mg_close_gen (NULL, sess->res->banlist_window);
+
 	if (sess->res->input_text)
 		free (sess->res->input_text);
 
