@@ -89,7 +89,7 @@ static GtkWidget *servlist_open_edit (GtkWidget *parent, ircnet *net);
 
 static const char *pages[]=
 {
-	"UTF-8",
+	"UTF-8 (Unicode)",
 	"ISO-8859-15 (Western Europe)",
 	"ISO-8859-2 (Central Europe)",
 	"ISO-8859-7 (Greek)",
@@ -269,7 +269,7 @@ servlist_addnet_cb (GtkWidget *item, GtkTreeView *treeview)
 #ifdef WIN32
 	/* Windows gets UTF-8 for new users. Unix gets "System Default",
 		which is often UTF-8 anyway! */
-	net->encoding = strdup ("UTF-8");
+	net->encoding = strdup ("UTF-8 (Unicode)");
 #endif
 	servlist_server_add (net, "newserver/6667");
 
@@ -1226,7 +1226,7 @@ servlist_open_networks (void)
 
 	servlist = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (servlist), 4);
-	gtk_window_set_title (GTK_WINDOW (servlist), _("XChat: Server List"));
+	gtk_window_set_title (GTK_WINDOW (servlist), _("XChat: Network List"));
 	gtk_window_set_default_size (GTK_WINDOW (servlist), win_width, win_height);
 	gtk_window_set_position (GTK_WINDOW (servlist), GTK_WIN_POS_MOUSE);
 	gtk_window_set_role (GTK_WINDOW (servlist), "servlist");
@@ -1363,13 +1363,13 @@ servlist_open_networks (void)
 								NULL);
 
 	checkbutton_skip =
-		gtk_check_button_new_with_mnemonic (_("Skip server list on startup"));
+		gtk_check_button_new_with_mnemonic (_("Skip network list on startup"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_skip),
 											prefs.slist_skip);
 	g_signal_connect (G_OBJECT (checkbutton_skip), "toggled",
 							G_CALLBACK (no_servlist), 0);
 	gtk_widget_show (checkbutton_skip);
-	gtk_table_attach (GTK_TABLE (table4), checkbutton_skip, 0, 1, 1, 2,
+	gtk_table_attach (GTK_TABLE (table4), checkbutton_skip, 0, 2, 1, 2,
 							(GtkAttachOptions) (GTK_FILL),
 							(GtkAttachOptions) (0), 0, 0);
 
